//----------------------------------------------------------------------------
//  The Frogs Of War By Linus Sphinx (c)Copyright 2005 all rights reserved
//----------------------------------------------------------------------------
//
//    This file is part of Linus Sphinx's Frogs Of War II
//
//    Frogs Of War II is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Frogs Of War II is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Frogs Of War II; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//--------------------------------------------------------------------------------
// top ten score list functions 
//--------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>
#include <time.h>
#include <SDL.h>
#include <SDL_mixer.h>
#include "frogs.h"
#include "racket.h"
#include "util.h"
#include "sprite.h"
#include "player.h"
#include "tank.h"
#include "room.h"
#include "helm.h"
#include "chardisp.h"
#include "topten.h"

TOPTEN Topten[ TENSIZE ];
TOPTEN Highscore = { " ", 0l, 0 };

short CurrentChar = 0;

//--------------------------------------------------------------------------------
// load a top ten score file - format is TENSIZE array of TOPTEN in binary
//--------------------------------------------------------------------------------
void toptenload( void )
{
	int tfile, i; 
	off_t actual;
		 
	tfile = open( "topten",  O_CREAT|O_RDWR, S_IRGRP |  S_IWGRP | S_IRUSR | S_IWUSR );
	if ( tfile > 1  )
	{
		actual = read( tfile, &Topten, sizeof(Topten));
		if ( actual < sizeof( Topten )) // bad or new file
		{
//			fprintf( stderr, "new top ten\n");
			strcpy( Topten[ 0 ].name, "SPHINXTER" ); 
			Topten[ 0 ].bestime =  1922000;
			Topten[ 0 ].score = 13;
			for ( i = 1; i < TENSIZE; i++ )
			{
				strcpy( Topten[ i ].name, "" );
				Topten[ i ].bestime = 0xFFFFFFFF;
				Topten[ i ].score = 0;
			}
		}
		close( tfile);
	}
}

//--------------------------------------------------------------------------------
// top ten eligibility test
//--------------------------------------------------------------------------------
short topteneligible()
{
	short retval = 0;
	if ( Topten[ TENSIZE - 1 ].bestime > Highscore.bestime )
		retval++;
	return( retval );
}

//--------------------------------------------------------------------------------
// top ten qsort callback
//--------------------------------------------------------------------------------
int toptencmp( const TOPTEN *t1, const TOPTEN *t2 )
{
	short ret = 0;

	if ( t1->bestime < t2->bestime  ) 
		ret = -1;
	if ( t1->bestime > t2->bestime ) 
		ret = 1;
	return ret;
}

//--------------------------------------------------------------------------------
// write a top ten score file in sorted order 
//--------------------------------------------------------------------------------
void toptensave( void )
{
	int tfile; 
	strcpy( Topten[ TENSIZE - 1 ].name, Highscore.name );
	Topten[ TENSIZE - 1 ].bestime = Highscore.bestime;
	Topten[ TENSIZE - 1 ].score = Highscore.score;
	qsort( Topten, TENSIZE, sizeof(TOPTEN), (void *)toptencmp );
	tfile = open( "topten",  O_CREAT|O_RDWR, S_IRGRP |  S_IWGRP | S_IRUSR | S_IWUSR );
	write( tfile, &Topten, sizeof(Topten));
	close( tfile );
}

//--------------------------------------------------------------------------------
// show the  top ten score list
//--------------------------------------------------------------------------------
void toptenshow( SDL_Surface *Screen ) 
{
	int i;
	char scorestr[ 27 ] = "";
	RECT screct = {0,0,SCREENWIDTH,SCREENHEIGHT};

	SDL_SetClipRect( Screen, &screct );
	loadbmp( Screen, "bmp/frame.bmp", 1 );

	for ( i = 0; i < TENSIZE; i++ )
	{
		if ( Topten[ i ].score == 0 )
			continue;
		sprintf( scorestr, "%-10s  %02lu:%02lu:%02lu  %d", 
		Topten[ i ].name,
		(((Topten[i].bestime / 18) / 60) / 60) / 60 % 60, ((Topten[i].bestime / 18) / 60) / 60 % 60, ((Topten[i].bestime / 18 ) / 60 )% 60,
		Topten[ i ].score );
//		fprintf( stderr, "%s\n", scorestr );
		chardispdrawxy( Screen, 37, (i * ( LEADING + CHARHIGH - 1)) + 32, scorestr, -1 );
	}
	SDL_Flip( Screen );
}

//--------------------------------------------------------------------------------
// collect a name and add/replace a top ten entry
//--------------------------------------------------------------------------------
int toptentry( SDL_Surface *Screen )
{
	int saved = 0;
	char topscore[ TOPSCOREMAXLEN ];
	SDL_Event event;

	sprintf( topscore, "%-10s  %02lu:%02lu:%02lu  %d",
		Highscore.name,
		(((Highscore.bestime / 18) / 60) / 60) / 60 % 60, ((Highscore.bestime / 18) / 60) / 60 % 60 , ((Highscore.bestime / 18 ) / 60 )% 60,
		Highscore.score );
	chardispdrawxy( Screen, 31, 210, topscore, CurrentChar );

	SDL_PollEvent( &event );
	switch( event.type )
	{
		case SDL_JOYBUTTONDOWN:
			switch(event.jbutton.button)
			{
				case GP2X_BUTTON_UP:
					if ( Highscore.name[ CurrentChar ] < 'A' || Highscore.name[ CurrentChar ] > 'Z' )
						Highscore.name[ CurrentChar ] = 'A';
					else if ( Highscore.name[ CurrentChar ] < 'Z' )
						Highscore.name[ CurrentChar ]++;
					else if ( Highscore.name[ CurrentChar ] == 'Z' )
						Highscore.name[ CurrentChar ] = ' ';
					break;
				case GP2X_BUTTON_DOWN:
					if ( Highscore.name[ CurrentChar ] < 'A' || Highscore.name[ CurrentChar ] > 'Z' )
						Highscore.name[ CurrentChar ] = 'Z';
					else if ( Highscore.name[ CurrentChar ] > 'A' )
						Highscore.name[ CurrentChar ]--;
					else if ( Highscore.name[ CurrentChar ] == 'A' )
						Highscore.name[ CurrentChar ] = ' ';
					break;
				case GP2X_BUTTON_RIGHT:
					if ( CurrentChar < NAMELEN - 1)
					{
						CurrentChar++;
						Highscore.name[ CurrentChar ] = ' ';
					}
					break;
				case GP2X_BUTTON_LEFT:
					if ( CurrentChar > 0 )
					{
						CurrentChar--;
						Highscore.name[ CurrentChar ] = ' ';
					}
					break;
				case GP2X_BUTTON_B:
					toptensave();
					saved = 1;
					break;
				case GP2X_BUTTON_R:
					nextrack();
				case GP2X_BUTTON_X:
				case GP2X_BUTTON_A:
				case GP2X_BUTTON_Y:
				case GP2X_BUTTON_SELECT:
				case GP2X_BUTTON_START:
				case GP2X_BUTTON_L:
				case GP2X_BUTTON_CLICK:
					break;
				case GP2X_BUTTON_VOLUP:
					racketvolume( 8 );
					break;
				case GP2X_BUTTON_VOLDOWN:
					racketvolume( -8 );
				default:
					break;
			}
		case SDL_KEYDOWN:
			switch( event.key.keysym.sym )
			{
				case SDLK_b:
					toptensave();
					saved++;
					break;
				case SDLK_F6:
					racketvolume( -8 );
					break;
				case SDLK_F7:
					racketvolume( 8 );
					break;
				case SDLK_r:
					nextrack();
					break;
				case SDLK_UP:
					if ( Highscore.name[ CurrentChar ] < 'A' || Highscore.name[ CurrentChar ] > 'Z' )
						Highscore.name[ CurrentChar ] = 'A';
					else if ( Highscore.name[ CurrentChar ] < 'Z' )
						Highscore.name[ CurrentChar ]++;
					else if ( Highscore.name[ CurrentChar ] == 'Z' )
						Highscore.name[ CurrentChar ] = ' ';
					break;
				case SDLK_DOWN:
					if ( Highscore.name[ CurrentChar ] < 'A' || Highscore.name[ CurrentChar ] > 'Z' )
						Highscore.name[ CurrentChar ] = 'Z';
					else if ( Highscore.name[ CurrentChar ] > 'A' )
						Highscore.name[ CurrentChar ]--;
					else if ( Highscore.name[ CurrentChar ] == 'A' )
						Highscore.name[ CurrentChar ] = ' ';
					break;
				case SDLK_LEFT:
					if ( CurrentChar > 0 )
					{
						CurrentChar--;
						Highscore.name[ CurrentChar ] = ' ';
					}
					break;
				case SDLK_RIGHT:
					if ( CurrentChar < NAMELEN - 1)
					{
						CurrentChar++;
						Highscore.name[ CurrentChar ] = ' ';
					}
				default:
					break;
			}
		default:
			break;
	}
	return( saved );
}

//--------------------------------------------------------------------------------
// how fast was he going
//--------------------------------------------------------------------------------
void toptenstoptheclock( unsigned long best, int top )
{
	Highscore.bestime = best; 
	Highscore.score = top;
}

//--------------------------------------------------------------------------------
// eof
//--------------------------------------------------------------------------------
